/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.dde.Clipboard1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "clipboard1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Clipboard1Adaptor
 */

Clipboard1Adaptor::Clipboard1Adaptor(MainWindow *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Clipboard1Adaptor::~Clipboard1Adaptor()
{
    // destructor
}

bool Clipboard1Adaptor::clipboardVisible() const
{
    // get the value of property clipboardVisible
    return qvariant_cast< bool >(parent()->property("clipboardVisible"));
}

void Clipboard1Adaptor::Hide()
{
    // handle method call org.deepin.dde.Clipboard1.Hide
    parent()->Hide();
}

void Clipboard1Adaptor::Show()
{
    // handle method call org.deepin.dde.Clipboard1.Show
    parent()->Show();
}

void Clipboard1Adaptor::Toggle()
{
    // handle method call org.deepin.dde.Clipboard1.Toggle
    parent()->Toggle();
}


#include "moc_clipboard1adaptor.cpp"
