/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.deepin.dde.ClipboardLoader1.xml -p ClipboardLoaderInterface
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CLIPBOARDLOADERINTERFACE_H
#define CLIPBOARDLOADERINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.dde.ClipboardLoader1
 */
class ComDeepinDdeClipboardLoaderInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.dde.ClipboardLoader1"; }

public:
    ComDeepinDdeClipboardLoaderInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~ComDeepinDdeClipboardLoaderInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> dataReborned(const QByteArray &buf)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(buf);
        return asyncCallWithArgumentList(QStringLiteral("dataReborned"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void dataComing(const QByteArray &buf);
};

namespace com {
  namespace deepin {
    namespace dde {
      typedef ::ComDeepinDdeClipboardLoaderInterface ClipboardLoader;
    }
  }
}
#endif
