// This file is autogenerated. Changes will be overwritten.
#include "HDCJJC5Y4L/moc_clipboardcontroller.cpp"
#include "HDCJJC5Y4L/moc_clipboarditem.cpp"
#include "HDCJJC5Y4L/moc_clipboardplugin.cpp"
#include "HDCJJC5Y4L/moc_quickpanelwidget.cpp"
#include "KAGPCZQMV5/moc_commoniconbutton.cpp"
#include "KAGPCZQMV5/moc_singlequickpanel.cpp"
#include "KAGPCZQMV5/moc_tipswidget.cpp"
